# coding=utf-8

from __future__ import unicode_literals

import json
import pickle
import time

from ..exception import MyJSONDecodeError

__all__ = ['ZhihuToken']


class ZhihuToken:
    def __init__(self, user_id, uid, access_token, expires_in, token_type,
                 refresh_token, cookie, lock_in=None, unlock_ticket=None):
        """
        知乎令牌。

        尽量不要直接使用这个类，而是用 :meth:`ZhihuToken.from_str` 或
        :meth:`ZhihuToken.form_dict` 或
        :meth:`ZhihuToken.from_file` 方法来构造。

        ..  note::

            本类仅在 :class:`.ZhihuClient` 类内使用，一般用户不需要了解。

        :param str|unicode user_id: 用户 ID
        :param int uid: 某个数字型用户 ID，貌似没啥用
        :param str|unicode access_token: 最重要的访问令牌
        :param int expires_in: 过期时间
        :param str|unicode token_type: 令牌类型
        :param str|unicode refresh_token: 刷新令牌
        :param str|unicode cookie: 登录成功后需要加上这段 Cookies
        :param int lock_in: 不知道用处
        :param str|unicode unlock_ticket: 不知道用处
        """
        self._create_at = time.time()
        self._user_id = uid
        self._uid = user_id
        self._access_token = access_token
        self._expires_in = expires_in
        self._expires_at = self._create_at + self._expires_in
        self._token_type = token_type
        self._refresh_token = refresh_token
        self._cookie = cookie

        # 以下两个属性暂时不知道用处
        self._lock_in = lock_in
        self._unlock_ticket = unlock_ticket

    @staticmethod
    def from_str(json_str):
        """
        从字符串读取 token。

        :param str|unicode json_str: 一个合法的代表知乎 Token 的 JSON 字符串
        :rtype: :class:`ZhihuToken`
        :raise ValueError: 提供的参数不合法时
        """
        try:
            return ZhihuToken.from_dict(json.loads(json_str))
        except (MyJSONDecodeError, ValueError):
            raise ValueError(
                '"{json_str}" is NOT a valid zhihu token json string.'.format(
                    json_str=json_str
                ))

    @staticmethod
    def from_dict(json_dict):
        """
        从字典读取 token。

        :param dict json_dict: 一个代表知乎 Token 的字典
        :rtype: :class:`ZhihuToken`
        :raise ValueError: 提供的参数不合法时
        """
        try:
            return ZhihuToken(**json_dict)
        except TypeError:
            raise ValueError(
                '"{json_dict}" is NOT a valid zhihu token json.'.format(
                    json_dict=json_dict
                ))

    @staticmethod
    def from_file(filename):
        """
        从文件读取 token。

        :param str|unicode filename: 文件名
        :rtype: :class:`ZhihuToken`
        """
        with open(filename, 'rb') as f:
            return pickle.load(f)

    def save(self, filename):
        """
        将 token 保存成文件。

        :param str|unicode filename: 文件名
        :return: 无返回值
        """
        with open(filename, 'wb') as f:
            pickle.dump(self, f)

    @property
    def user_id(self):
        """
        :return: 获取用户 ID
        :rtype: str
        """
        return self._user_id

    @property
    def type(self):
        """
        :return: 获取验证类型
        :rtype: str
        """
        return self._token_type

    @property
    def token(self):
        """
        :return: 获取访问令牌
        :rtype: str
        """
        return self._access_token



# coding=utf-8

from __future__ import unicode_literals

from .base import Base
from .generator import generator_of
from .other import other_obj
from .normal import normal_attr
from .urls import (
    COLLECTION_DETAIL_URL,
    COLLECTION_ANSWERS_URL,
    COLLECTION_COMMENTS_URL,
    COLLECTION_FOLLOWERS_URL,
)


class Collection(Base):
    def __init__(self, cid, cache, session):
        super(Collection, self).__init__(cid, cache, session)

    def _build_url(self):
        return COLLECTION_DETAIL_URL.format(self.id)

    # ---- simple info -----

    @property
    @normal_attr()
    def answer_count(self):
        return None

    @property
    @normal_attr()
    def created_time(self):
        return None

    @property
    @other_obj('people')
    def creator(self):
        return None

    @property
    @normal_attr()
    def comment_count(self):
        return None

    @property
    @normal_attr()
    def description(self):
        return None

    @property
    @normal_attr()
    def follower_count(self):
        return None

    @property
    @normal_attr()
    def id(self):
        return self._id

    @property
    @normal_attr()
    def is_public(self):
        return None

    @property
    @normal_attr()
    def title(self):
        return None

    @property
    @normal_attr()
    def updated_time(self):
        return None

    # ----- generators -----

    @property
    @generator_of(COLLECTION_ANSWERS_URL)
    def answers(self):
        return None

    @property
    @generator_of(COLLECTION_COMMENTS_URL)
    def comments(self):
        return None

    @property
    @generator_of(COLLECTION_FOLLOWERS_URL, 'people')
    def followers(self):
        """
        ..  warning:: 注意！

            知乎的这个 API 有问题，返回一些之后会将 is_end 设置为 True，
            导致无法获取到所有的关注者。

            并且此问题在知乎官方 Android APP 上也存在。你可以试着
            找个很多人关注的收藏夹，然后查看关注者，一直往下拉。
            大概加载 100 - 200（不固定，有时候一个都出不来）
            之后就没法往下刷了。

            起码在我这个地区是这样的。欢迎各路少侠反馈。

        """
        # TODO: collection.followers 这个 API 不稳定
        return None



from __future__ import unicode_literals

from zhihu_oauth import (Answer, Article, Collection, Column, People, Question,
                         Topic)

from .client_test_base import ZhihuClientClassTest


class TestZhihuClientFromURL(ZhihuClientClassTest):
    def test_correct_answer_url(self):
        url_list = [
            # normal url
            'https://www.zhihu.com/question/42248369/answer/94150403',
            # with http scheme
            'http://www.zhihu.com/question/42248369/answer/94150403',
            # with a slash at end
            'https://www.zhihu.com/question/42248369/answer/94150403/',
            # with http and slash
            'http://www.zhihu.com/question/42248369/answer/94150403/',
            # without scheme
            'www.zhihu.com/question/42248369/answer/94150403',
            # without scheme but has slash
            'www.zhihu.com/question/42248369/answer/94150403/'
        ]
        for url in url_list:
            obj = self.client.from_url(url)
            self.assertIsInstance(obj, Answer)
            self.assertEqual(obj.id, 94150403)

    def test_error_answer_url(self):
        url_list = [
            # no www
            'https://zhihu.com/question/42248369/answer/94150403',
            # no answer part
            'https://www.zhihu.com/question/42248369',
            # just a num
            '94150403',
            # answer id not int
            'https://www.zhihu.com/question/42248369/answer/error',
            # question id not int
            'https://www.zhihu.com/question/error/answer/94150403',
            # question spelling error
            'https://www.zhihu.com/question_er/42248369/answer/94150403',
            # answer spelling error
            'https://www.zhihu.com/question/42248369/answer_er/94150403',
            # more than a invalid url
            'some text https://zhihu.com/question/42248369/answer/94150403',
            # more than a invalid url
            'https://www.zhihu.com/question/error/answer/94150403 some text',
        ]

        for url in url_list:
            with self.assertRaises(ValueError):
                obj = self.client.from_url(url)
                if not isinstance(obj, Answer):
                    raise ValueError

    def test_correct_article_url(self):
        url_list = [
            # normal url
            'http://zhuanlan.zhihu.com/p/20597723',
            # with https scheme
            'https://zhuanlan.zhihu.com/p/20597723',
            # with a slash at end
            'http://zhuanlan.zhihu.com/p/20597723/',
            # with http and slash
            'https://zhuanlan.zhihu.com/p/20597723/',
            # without scheme
            'zhuanlan.zhihu.com/p/20597723',
            # without scheme but has slash
            'zhuanlan.zhihu.com/p/20597723/'
        ]
        for url in url_list:
            obj = self.client.from_url(url)
            self.assertIsInstance(obj, Article)
            self.assertEqual(obj.id, 20597723)

    def test_error_article_url(self):
        url_list = [
            # no p - become a column url
            'http://zhuanlan.zhihu.com/20597723',
            # just a num
            '20597723',
            # article id not int
            'http://zhuanlan.zhihu.com/p/error',
            # p spelling errors
            'http://zhuanlan.zhihu.com/p_er/20597723',
            # more than a invalid url
            'some text http://zhuanlan.zhihu.com/p/20597723',
            # more than a invalid url
            'http://zhuanlan.zhihu.com/p/20597723 some text',
        ]

        for url in url_list:
            with self.assertRaises(ValueError):
                obj = self.client.from_url(url)
                if not isinstance(obj, Article):
                    raise ValueError

    def test_correct_collection_url(self):
        url_list = [
            # normal url
            'https://www.zhihu.com/collection/28698204',
            # with http scheme
            'http://www.zhihu.com/collection/28698204',
            # with a slash at end
            'https://www.zhihu.com/collection/28698204/',
            # with http and slash
            'http://www.zhihu.com/collection/28698204/',
            # without scheme
            'www.zhihu.com/collection/28698204',
            # without scheme but has slash
            'www.zhihu.com/collection/28698204/'
        ]
        for url in url_list:
            obj = self.client.from_url(url)
            self.assertIsInstance(obj, Collection)
            self.assertEqual(obj.id, 28698204)

    def test_error_collection_url(self):
        url_list = [
            # just a num
            '28698204',
            # collection id not int
            'https://www.zhihu.com/collection/error',
            # collection spellling error
            'https://www.zhihu.com/collection_er/28698204',
            # more than a invalid url
            'some text https://www.zhihu.com/collection/28698204',
            # more than a invalid url
            'https://www.zhihu.com/collection/28698204 some text',
        ]

        for url in url_list:
            with self.assertRaises(ValueError):
                self.client.from_url(url)

    def test_correct_column_url(self):
        url_list = [
            # normal url
            'http://zhuanlan.zhihu.com/o0v0o',
            # with https scheme
            'https://zhuanlan.zhihu.com/o0v0o',
            # with a slash at end
            'http://zhuanlan.zhihu.com/o0v0o/',
            # with https and slash
            'https://zhuanlan.zhihu.com/o0v0o/',
            # without scheme
            'zhuanlan.zhihu.com/o0v0o',
            # without scheme but has slash
            'zhuanlan.zhihu.com/o0v0o/'
        ]
        for url in url_list:
            obj = self.client.from_url(url)
            self.assertIsInstance(obj, Column)
            self.assertEqual(obj.id, 'o0v0o')

    def test_error_column_url(self):
        url_list = [
            # just a id
            'o0v0o',
            # id has slash
            'http://zhuanlan.zhihu.com/o0v/0o',
            # more than a invalid url
            'some text http://zhuanlan.zhihu.com/o0v0o',
            # more than a invalid url
            'http://zhuanlan.zhihu.com/o0v0o some text',
        ]

        for url in url_list:
            with self.assertRaises(ValueError):
                self.client.from_url(url)

    def test_correct_people_url(self):
        url_list = [
            # normal url
            'https://www.zhihu.com/people/7sdream',
            # with http scheme
            'http://www.zhihu.com/people/7sdream',
            # with a slash at end
            'https://www.zhihu.com/people/7sdream/',
            # with http and slash
            'http://www.zhihu.com/people/7sdream/',
            # without scheme
            'www.zhihu.com/people/7sdream',
            # without scheme but has slash
            'www.zhihu.com/people/7sdream/'
        ]

        for url in url_list:
            obj = self.client.from_url(url)
            self.assertIsInstance(obj, People)
            self.assertEqual(obj.id, '7sdream')

    def test_error_people_url(self):
        url_list = [
            # just a id
            '7sdream',
            # id has slash
            'https://www.zhihu.com/people/7sdr/eam',
            # spelling error
            'https://www.zhihu.com/people_er/7sdream',
            # more than a invalid url
            'some text https://www.zhihu.com/people/7sdream',
            # more than a invalid url
            'https://www.zhihu.com/people/7sdream some text',
        ]

        for url in url_list:
            with self.assertRaises(ValueError):
                self.client.from_url(url)

    def test_correct_question_url(self):
        url_list = [
            # normal url
            'https://www.zhihu.com/question/42248369',
            # with http scheme
            'http://www.zhihu.com/question/42248369',
            # with a slash at end
            'https://www.zhihu.com/question/42248369/',
            # with http and slash
            'http://www.zhihu.com/question/42248369/',
            # without scheme
            'www.zhihu.com/question/42248369',
            # without scheme but has slash
            'www.zhihu.com/question/42248369/'
        ]

        for url in url_list:
            obj = self.client.from_url(url)
            self.assertIsInstance(obj, Question)
            self.assertEqual(obj.id, 42248369)

    def test_error_question_url(self):
        url_list = [
            # just a id
            '42248369',
            # id not int
            'https://www.zhihu.com/question/error',
            # spelling error
            'https://www.zhihu.com/question_er/42248369',
            # has answer part - become an answer
            'https://www.zhihu.com/question/42248369/answer/123456',
            # more than a invalid url
            'some text https://www.zhihu.com/question/42248369',
            # more than a invalid url
            'https://www.zhihu.com/question/42248369 some text',
        ]

        for url in url_list:
            with self.assertRaises(ValueError):
                obj = self.client.from_url(url)
                if not isinstance(obj, Question):
                    raise ValueError

    def test_correct_topic_url(self):
        url_list = [
            # normal url
            'https://www.zhihu.com/topic/19550434',
            # with http scheme
            'http://www.zhihu.com/topic/19550434',
            # with a slash at end
            'https://www.zhihu.com/topic/19550434/',
            # with http and slash
            'https://www.zhihu.com/topic/19550434/',
            # without scheme
            'www.zhihu.com/topic/19550434',
            # without scheme but has slash
            'www.zhihu.com/topic/19550434/'
        ]

        for url in url_list:
            obj = self.client.from_url(url)
            self.assertIsInstance(obj, Topic)
            self.assertEqual(obj.id, 19550434)

    def test_error_topic_url(self):
        url_list = [
            # just a id
            '19550434',
            # id not int
            'https://www.zhihu.com/topic/error',
            # spelling error
            'https://www.zhihu.com/topic_er/19550434',
            # more than a invalid url
            'some text https://www.zhihu.com/topic/19550434',
            # more than a invalid url
            'https://www.zhihu.com/topic/19550434 some text',
        ]

        for url in url_list:
            with self.assertRaises(ValueError):
                obj = self.client.from_url(url)
                if not isinstance(obj, Topic):
                    raise ValueError



# coding=utf-8

from __future__ import unicode_literals

from .base import Base
from .generator import generator_of
from .other import other_obj
from .normal import normal_attr
from .streaming import streaming
from .utils import common_save
from .urls import (
    ARTICLE_DETAIL_URL,
    ARTICLE_COMMENTS_URL,
)

__all__ = ['Article']


class Article(Base):
    def __init__(self, aid, cache, session):
        super(Article, self).__init__(aid, cache, session)

    def _build_url(self):
        return ARTICLE_DETAIL_URL.format(self.id)

    # ----- simple info -----

    @property
    @other_obj('people')
    def author(self):
        return None

    @property
    @streaming()
    def can_comment(self):
        """
        ..  seealso:: :any:`Answer.can_comment`
        """
        return None

    @property
    @other_obj()
    def column(self):
        return None

    @property
    @normal_attr()
    def comment_count(self):
        return None

    @property
    @normal_attr()
    def comment_permission(self):
        """
        ..  seealso:: :any:`Answer.comment_permission`
        """
        return None

    @property
    @normal_attr()
    def content(self):
        return None

    @property
    @normal_attr()
    def excerpt(self):
        return None

    @property
    @normal_attr()
    def id(self):
        return self._id

    @property
    @normal_attr()
    def image_url(self):
        return None

    @property
    @streaming(use_cache=False)
    def suggest_edit(self):
        """
        ..  seealso:: :any:`Answer.suggest_edit`
        """
        return None

    @property
    @normal_attr()
    def title(self):
        return None

    @property
    @normal_attr('updated')
    def updated_time(self):
        return None

    @property
    @normal_attr()
    def voteup_count(self):
        return None

    # ----- generators -----

    @property
    @generator_of(ARTICLE_COMMENTS_URL)
    def comments(self):
        return None

    # TODO: article.voters, API 接口未知

    # ----- other operate -----

    def save(self, path='.', filename=None, invalid_chars=None):
        """
        除了默认文件名是文章标题外，和 :any:`Answer.save` 完全一致。

        ..  seealso:: :any:`Answer.save`

        ..  note:: TIPS

            建议的使用方法：

            ..  code-block:: python

                for article in column.articles:
                    print(article.title)
                    article.save(column.title)

        """
        if self._cache is None:
            self._get_data()
        common_save(path, filename, self.content, self.title, invalid_chars)



# coding=utf-8

from __future__ import unicode_literals

import hashlib
import hmac
import time

__all__ = ['login_signature']


def login_signature(data, secret):
    """
    为登录请求附加签名。

    :param dict data: POST 数据
    :param str|unicode secret: APP SECRET
    :return: 经过签名后的 dict， 增加了 timestamp 和 signature 两项
    """
    data['timestamp'] = str(int(time.time()))

    params = ''.join([
        data['grant_type'],
        data['client_id'],
        data['source'],
        data['timestamp'],
    ])

    data['signature'] = hmac.new(
        secret.encode('utf-8'),
        params.encode('utf-8'),
        hashlib.sha1
    ).hexdigest()



# coding=utf-8

from __future__ import unicode_literals

from .people import People
from .urls import (
    ANSWER_CANCEL_THANKS_URL,
    ANSWER_CANCEL_UNHELPFUL_URL,
    ANSWER_COLLECT_URL,
    ANSWER_DETAIL_URL,
    ANSWER_THANKS_URL,
    ANSWER_UNHELPFUL_URL,
    ANSWER_VOTERS_URL,
    ARTICLE_DETAIL_URL,
    ARTICLE_VOTE_URL,
    BLOCK_PEOPLE_URL,
    CANCEL_BLOCK_PEOPLE_URL,
    COLLECTION_CANCEL_FOLLOW_URL,
    COLLECTION_DETAIL_URL,
    COLLECTION_FOLLOWERS_URL,
    COLUMN_CANCEL_FOLLOW_URL,
    COLUMN_FOLLOWERS_URL,
    COMMENT_CANCEL_VOTE_URL,
    COMMENT_DETAIL_URL,
    COMMENT_VOTE_URL,
    PEOPLE_CANCEL_FOLLOWERS_URL,
    PEOPLE_FOLLOWERS_URL,
    PEOPLE_FOLLOWING_COLLECTIONS_URL,
    QUESTION_CANCEL_FOLLOWERS_URL,
    QUESTION_FOLLOWERS_URL,
    SELF_DETAIL_URL,
    SEND_COMMENT_URL,
    SEND_MESSAGE_URL,
    TOPIC_CANCEL_FOLLOW_URL,
    TOPIC_FOLLOWERS_URL,
)
from .generator import generator_of
from .utils import get_result_or_error

__all__ = ['Me']


class Me(People):
    def __init__(self, pid, cache, session):
        """
        是 :any:`People` 的子类，表示当前登录的用户。
        除了提供用户的基本信息外，还提供各种用户操作
        （点赞，评论，收藏，私信、删除等）。

        ..  inheritance-diagram:: Me

        ..  seealso:: :class:`People`

        """
        super(Me, self).__init__(pid, cache, session)

    def _build_url(self):
        return SELF_DETAIL_URL

    # ----- generators -----

    @property
    @generator_of(PEOPLE_FOLLOWING_COLLECTIONS_URL, 'collection')
    def following_collections(self):
        """
        ..  warning:: 注意

            这一方法是 :any:`Me` 类独有的，其父类 :any:`People` 类没有此方法。

            根本原因是知乎并不允许获取除自己（登录用户）以外用户关注的专栏，
            至于为什么，我哪知道呀QAQ
        """
        return None

    def vote(self, what, op='up'):
        """
        投票操作。也就是赞同，反对，或者清除（取消赞同和反对）。

        操作对象可以是答案，文章和评论。

        :param what: 要点赞的对象，可以是 :any:`Answer` 或 :any:`Article`
          或 :any:`Comment` 对象。
        :param str|unicode op: 对于答案可取值 'up', 'down', 'clear'，
          分别表示赞同、反对和清除。
          对于文章和文章，只能取 'up' 和 'clear'。默认值是 'up'。
        :return: 表示结果的二元组，第一项表示是否成功，第二项表示原因。
        :rtype: (bool, str)
        :raise: :any:`UnexpectedResponseException`
          当服务器回复和预期不符，不知道是否成功时。
        """
        from . import Answer, Article, Comment
        if isinstance(what, Answer):
            if op not in {'up', 'down', 'clear'}:
                raise ValueError(
                    'Operate must be up, down or clear for Answer.')
            return self._common_vote(ANSWER_VOTERS_URL, what, op)
        elif isinstance(what, Article):
            if op not in {'up', 'clear'}:
                raise ValueError('Operate must be up or clear for Article')
            return self._common_vote(ARTICLE_VOTE_URL, what, op)
        elif isinstance(what, Comment):
            if op not in {'up', 'clear'}:
                raise ValueError('Operate must be up or clear for Comment')
            return self._common_click(what, op == 'clear', COMMENT_VOTE_URL,
                                      COMMENT_CANCEL_VOTE_URL)
        else:
            raise TypeError(
                'Unable to voteup a {0}.'.format(what.__class__.__name__))

    def thanks(self, answer, thanks=True):
        """
        感谢或者取消感谢答案。

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        :param  Answer answer: 要感谢的答案
        :param bool thanks: 如果是想取消感谢，请设置为 False
        """
        from .answer import Answer
        if not isinstance(answer, Answer):
            raise TypeError('This method only accept Answer object.')
        return self._common_click(answer, not thanks,
                                  ANSWER_THANKS_URL, ANSWER_CANCEL_THANKS_URL)

    def unhelpful(self, answer, unhelpful=True):
        """
        给答案点没有帮助，或者取消没有帮助。

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        :param Answer answer: 要操作的答案
        :param bool unhelpful: 如果是想撤销没有帮助，请设置为 False
        """
        from .answer import Answer
        if not isinstance(answer, Answer):
            raise TypeError('This method only accept Answer object.')
        return self._common_click(answer, not unhelpful,
                                  ANSWER_UNHELPFUL_URL,
                                  ANSWER_CANCEL_UNHELPFUL_URL)

    def follow(self, what, follow=True):
        """
        关注或者取消关注问题/话题/用户/专栏/收藏夹。

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        :param what: 操作对象
        :param follow: 要取消关注的话把这个设置成 False
        """
        from . import Question, Topic, People, Column, Collection
        if isinstance(what, Question):
            return self._common_click(what, not follow,
                                      QUESTION_FOLLOWERS_URL,
                                      QUESTION_CANCEL_FOLLOWERS_URL)
        elif isinstance(what, Topic):
            return self._common_click(what, not follow, TOPIC_FOLLOWERS_URL,
                                      TOPIC_CANCEL_FOLLOW_URL)
        elif isinstance(what, People):
            what._get_data()
            return self._common_click(what, not follow, PEOPLE_FOLLOWERS_URL,
                                      PEOPLE_CANCEL_FOLLOWERS_URL)
        elif isinstance(what, Column):
            return self._common_click(what, not follow, COLUMN_FOLLOWERS_URL,
                                      COLUMN_CANCEL_FOLLOW_URL)
        elif isinstance(what, Collection):
            return self._common_click(what, not follow,
                                      COLLECTION_FOLLOWERS_URL,
                                      COLLECTION_CANCEL_FOLLOW_URL)
        else:
            raise TypeError(
                'Unable to follow a {0}.'.format(what.__class__.__name__))

    def block(self, what, block=True):
        """
        屏蔽用户/话题

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        :param what: 操作对象，用户或话题
        :param bool block: 如果要取消屏蔽请设置为 False
        """
        from . import People
        if isinstance(what, People):
            return self._common_block(what, not block, BLOCK_PEOPLE_URL,
                                      CANCEL_BLOCK_PEOPLE_URL)
        else:
            raise TypeError(
                'Unable to block a {0}.'.format(what.__class__.__name__))

    def collect(self, answer, collection, collect=True):
        """
        收藏答案进收藏夹。

        ..  warning::

            就算你提供的是别人的收藏夹也会返回成功……但是操作其实是无效的

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        :param Answer answer: 要收藏的答案
        :param Collection collection: 要加入哪个收藏夹
        :param bool collect: 如果想要取消收藏请设置为 False
        """
        from . import Answer, Collection
        if not isinstance(answer, Answer):
            raise TypeError('Unable to add a {0} to collection.'.format(
                answer.__class__.__name__))
        if not isinstance(collection, Collection):
            raise TypeError('Unable add answer to a {0}.'.format(
                collection.__class__.__name__))
        if collect:
            data = {'add_collections': collection.id}
        else:
            data = {'remove_collections': collection.id}
        url = ANSWER_COLLECT_URL.format(answer.id)
        res = self._session.put(url, data=data)
        return get_result_or_error(url, res)

    def message(self, who, content):
        """
        发送私信。

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        :param  People who: 接收者
        :param str|unicode content: 私信内容
        """
        from . import People
        if not isinstance(who, People):
            raise TypeError(
                'Unable to send message to {0}'.format(who.__class__.__name__))
        _ = who.name
        data = {
            'receiver_id': who.id,
            'content': content,
        }
        res = self._session.post(SEND_MESSAGE_URL, data=data)
        return get_result_or_error(SEND_MESSAGE_URL, res)

    def comment(self, what, content, parent=None):
        """
        向答案发送评论

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        ..  warning:: 奇怪

            让我很诧异的是，就算「想要回复的评论」不属于「想要评论的主体」，
            知乎的 API 也会返回执行成功。而且经过测试，这条回复真的有效，
            会出现在评论主体的评论列表里。暂时不知道被评论用户的会不会收到消息。

            另外，莫名其妙的还可以回复自己的评论……

        :param what: 向哪里发送评论，可以是 :any:`Answer`, :any:`Article`
          :any:`Question`, :any:`Collection`
        :param str|unicode content: 评论内容
        :param Comment parent: 想要回复的评论，默认值为 None，则为正常的添加评论
        """
        from . import Answer, Article, Question, Collection, Comment
        data = {'content': content}
        if parent is not None:
            if not isinstance(parent, Comment):
                raise TypeError(
                    'parent comment must be Comment object, {0} given.'.format(
                        parent.__class__.__name__))
            data.update(comment_id=parent.id)
        if isinstance(what, (Answer, Article, Collection, Question)):
            data.update({'type': what.__class__.__name__.lower(),
                         'resource_id': what.id})
        else:
            raise TypeError('Can\'t add comment to a {0}.'.format(
                what.__class__.__name__))
        res = self._session.post(SEND_COMMENT_URL, data=data)
        print(res.text)
        return get_result_or_error(SEND_COMMENT_URL, res)

    def delete(self, what):
        """
        删除……一些东西，目前可以删除答案，评论，收藏夹，文章。

        ..  seealso::

            返回值和可能的异常同 :any:`vote` 方法

        ..  warning::

            请注意，本方法没有经过完整的测试，加上删除操作不可撤销，
            所以使用时请谨慎。

        :param what: 要删除的对象，可以是 :any:`Answer`, :any:`Comment`,
          :any:`Collection`, :any:`Article`
        """
        from . import Answer, Comment, Collection, Article
        if isinstance(what, Answer):
            url = ANSWER_DETAIL_URL.format(what.id)
        elif isinstance(what, Comment):
            url = COMMENT_DETAIL_URL.format(what.id)
        elif isinstance(what, Collection):
            url = COLLECTION_DETAIL_URL.format(what.id)
        elif isinstance(what, Article):
            url = ARTICLE_DETAIL_URL.format(what.id)
        else:
            raise TypeError(
                'Can\'t delete a {0}.'.format(what.__class__.__name__))
        res = self._session.delete(url)
        return get_result_or_error(url, res)

    def _common_click(self, what, cancel, click_url, cancel_url):
        if cancel:
            method = 'DELETE'
            url = cancel_url.format(what.id, self.id)
        else:
            method = 'POST'
            url = click_url.format(what.id)
        res = self._session.request(method, url)
        return get_result_or_error(url, res)

    def _common_vote(self, url, what, op):
        data = {
            'voteup_count': 0,
            'voting': {'up': 1, 'down': -1, 'clear': 0}[op],
        }
        url = url.format(what.id)
        res = self._session.post(url, data=data)
        return get_result_or_error(url, res)

    def _common_block(self, what, cancel, block_url, cancel_url):
        _ = what.name
        if cancel:
            method = 'DELETE'
            data = None
            url = cancel_url.format(what.id)
        else:
            method = 'POST'
            data = {'people_id': what.id}
            url = block_url
        res = self._session.request(method, url, data=data)
        return get_result_or_error(url, res)



# coding=utf-8

from __future__ import unicode_literals

import importlib

from .streaming import StreamingJSON
from .utils import SimpleEnum
from ..exception import UnimplementedException

_verb_to_type_map = {
    'MEMBER_VOTEUP_ARTICLE': 'VOTEUP_ARTICLE',
    'ANSWER_VOTE_UP': 'VOTEUP_ANSWER',
    'QUESTION_FOLLOW': 'FOLLOW_QUESTION',
    'ANSWER_CREATE': 'CREATE_ANSWER',
    'QUESTION_CREATE': 'CREATE_QUESTION',
    'MEMBER_CREATE_ARTICLE': 'CREATE_ARTICLE',
    'TOPIC_FOLLOW': 'FOLLOW_TOPIC',
    'MEMBER_FOLLOW_COLUMN': 'FOLLOW_COLUMN',
    'MEMBER_FOLLOW_TOPIC': 'FOLLOW_TOPIC',
    'MEMBER_FOLLOW_COLLECTION': 'FOLLOW_COLLECTION',
    'MEMBER_FOLLOW_ROUNDTABLE': 'FOLLOW_ROUNDTABLE'
}

ActType = SimpleEnum([v for _, v in _verb_to_type_map.items()])
"""
ActType 是用于表示用户动态类型的枚举类，可供使用的常量有：

================= ================ =====================
常量名              说明              `target` 属性类型
================= ================ =====================
CREATE_ANSWER      回答问题          :any:`Answer`
CREATE_ARTICLE     发表文章          :any:`Article`
CREATE_QUESTION    提出问题          :any:`Question`
FOLLOW_COLLECTION  关注收藏夹        :any:`Collection`
FOLLOW_COLUMN      关注专栏          :any:`Column`
FOLLOW_QUESTION    关注问题          :any:`Question`
FOLLOW_ROUNDTABLE  关注圆桌          :any:`StreamingJSON`
FOLLOW_TOPIC       关注话题          :any:`Topic`
VOTEUP_ANSWER      赞同回答          :any:`Answer`
VOTEUP_ARTICLE     赞同文章          :any:`Article`
================= ================ =====================
"""


def _verb_to_type(verb):
    type_str = _verb_to_type_map.get(verb, None)
    if type_str is None:
        raise UnimplementedException(
            'Unknown activity type: {0}. '
            'Please send this error message to '
            'developer to get help.'.format(verb))
    return getattr(ActType, _verb_to_type_map[verb])


class Activity(object):
    def __new__(cls, data, session):
        if data['verb'].endswith('ROUNDTABLE'):
            data['type'] = ActType.FOLLOW_ROUNDTABLE
            return StreamingJSON(data)
        else:
            return super(Activity, cls).__new__(cls)

    def __init__(self, data, session):
        """
        表示用户的一条动态。

        :any:`type <Activity.type>` 属性标识了动态的类型，其取值及意义请参见
        :any:`ActType`。

        :any:`target <Activity.target>` 属性表示这次动态操作的目标，根据
        `type` 的不同，这个属性的类型也不同。但是基本都是
        `type` 的最后一个单词表示的类型，请看下面的例子。

        ..  note:: 举例

            ..  code-block:: python

                for act in me.activities:
                    if act.type == ActType.CREATE_ANSWER:
                        print(act.target.question.title)

            上面这段代码判断的类型是创建答案，此时 `act.target` 就是
            :any:`Answer` 类型，和 `CREATE_ANSWER` 的 `ANSWER` 对应。
        """
        self._data = data
        self._type = _verb_to_type(data['verb'])
        self._session = session
        self._get_target()

    @property
    def type(self):
        """
        动态的类型。

        ..  seealso:: :any:`ActType`
        """
        return self._type

    @property
    def target(self):
        """
        动态的操作目标。

        ..  seealso:: :any:`Activity.__init__`, :any:`ActType`
        """
        return self._target

    def _get_target(self):
        pos = self._type.rfind('_')
        if pos == -1:
            raise UnimplementedException('Unable to get class from type name')
        filename = self._type[pos + 1:].lower()
        class_name = filename.capitalize()
        module = importlib.import_module('.' + filename, 'zhihu_oauth.zhcls')
        cls = getattr(module, class_name)
        self._target = cls(self._data['target']['id'], self._data['target'],
                           self._session)



# coding=utf-8

from __future__ import unicode_literals

try:
    from json import JSONDecodeError as MyJSONDecodeError
except ImportError:
    MyJSONDecodeError = Exception
    """
    在 Py3 下是 json.JSONDecodeError。

    在 Py2 下是 BaseException，因为 Py2 的 json 模块没有提供解析异常。
    """

__all__ = [
    'UnexpectedResponseException',
    'GetDataErrorException',
    'NeedCaptchaException',
    'NeedLoginException',
    'IdMustBeIntException',
    'UnimplementedException',
    'MyJSONDecodeError',
]


class UnexpectedResponseException(Exception):
    def __init__(self, url, res, expect):
        """
        服务器回复了和预期格式不符的数据

        :param str|unicode url: 当前尝试访问的网址
        :param request.Response res: 服务器的回复
        :param str|unicode expect: 一个用来说明期望服务器回复的数据格式的字符串
        """
        self.url = url
        self.res = res
        self.expect = expect

    def __repr__(self):
        return 'Get an unexpected response when visit url ' \
               '"{self.url}", we expect "{self.expect}", ' \
               'but the response body is "{self.res.text}"'.format(self=self)

    __str__ = __repr__


class UnimplementedException(Exception):
    def __init__(self, what):
        """
        处理当前遇到的情况的代码还未实现，只是开发的时候用于占位

        ..  note:: 一般用户不用管这个异常

        :param str|unicode what: 用来描述当前遇到的情况
        """
        self.what = what

    def __repr__(self):
        return 'Meet a unimplemented station: {self.what}'.format(self=self)

    __str__ = __repr__


class GetDataErrorException(UnexpectedResponseException):
    def __init__(self, url, res, expect):
        """
        :class:`UnexpectedResponseException` 的子类，
        尝试获取服务器给出的错误信息。如果获取失败则显示父类的出错信息。

        ..  seealso:: :class:`UnexpectedResponseException`
        """
        super(GetDataErrorException, self).__init__(url, res, expect)
        try:
            self._reason = res.json()['error']['message']
        except (MyJSONDecodeError, KeyError):
            self._reason = None

    def __repr__(self):
        if self._reason:
            return 'A error happened when get data: {0}'.format(self._reason)
        else:
            base = super(GetDataErrorException, self).__repr__()
            return "Unknown error! " + base

    __str__ = __repr__


class NeedCaptchaException(Exception):
    def __init__(self):
        """
        登录过程需要验证码
        """
        pass

    def __repr__(self):
        return "Need a captcha to login, " \
               "please catch this exception and " \
               "use client.get_captcha() to get it."

    __str__ = __repr__


class NeedLoginException(Exception):
    def __init__(self, what):
        """
        使用某方法需要登录而当前客户端未登录

        :param str|unicode what: 当前试图调用的方法名
        """
        self.what = what

    def __repr__(self):
        return 'Need login to use the "{self.what}" method.'.format(self=self)

    __str__ = __repr__


class IdMustBeIntException(Exception):
    def __init__(self, func):
        """
        获取对应的知乎类时，试图传递不是整数型的 ID

        :param function func: 当前试图调用的方法名
        """
        self.func = func.__name__

    def __repr__(self):
        return "You must provide a integer id " \
               "to use function: {self.func}".format(self=self)

    __str__ = __repr__


class IgnoreErrorDataWarning(UserWarning):
    def __init__(self, message, *args, **kwargs):
        self._message = message
        super(IgnoreErrorDataWarning, self).__init__(*args, **kwargs)

    def __str__(self):
        return str(self._message)

    __repr__ = __str__


GetEmptyResponseWhenFetchData = IgnoreErrorDataWarning(
    "试图获取下一项时，服务器返回了空数据。"
    "如果您是在获取某用户的粉丝，那么您可能遇到了知乎 5020 限制。"
    "虽然不知道为什么，但是好像知乎限制 API 只能访问前 5020 个粉丝，"
    "我也很为难，但是这是知乎做的限制，突破不了呀。"
    "目前在遇到这个问题时只能当作获取完处理了。"
)



# coding=utf-8

from __future__ import unicode_literals

import functools
import importlib

__all__ = ['other_obj']


def other_obj(class_name=None, name_in_json=None):
    """

    本装饰器的作用为：

    1. 标识这个属性为另一个知乎对象。
    2. 自动从当前对象的数据中取出对应属性，构建成所需要的对象。

    生成对象流程如下：

    1. 尝试导入类名表示的类，如果获取失败则设为 :any:`Base` 类。
    2. 尝试从 ``cache`` 中获取用来建立对象的数据。失败转 3，成功转 5。
    3. 如果当前对象没有 ``data`` 则调用知乎 API 获取。
    4. 尝试从 ``data`` 中获取数据。失败则将数据设置为被装饰函数的返回值。
    5. 将获取到的数据作为 ``cache`` 构建第一步中的导入的知乎类对象。

    ..  seealso:: 关于 cache 和 data

        请看 :any:`Base` 类中的\ :any:`说明 <Base.__init__>`。

    :param class_name: 要生成的对象类名
    :param name_in_json: 属性在 JSON 里的键名。
    """
    def wrappers_wrapper(func):
        @functools.wraps(func)
        def wrapper(self, *args, **kwargs):
            cls_name = class_name or func.__name__
            cls_name = cls_name.capitalize()
            name_in_j = name_in_json or func.__name__
            file_name = '.' + cls_name.lower()

            try:
                module = importlib.import_module(file_name, 'zhihu_oauth.zhcls')
                cls = getattr(module, cls_name)
            except (ImportError, AttributeError):
                from .base import Base
                cls = Base

            if self._cache and name_in_j in self._cache:
                cache = self._cache[name_in_j]
            else:
                self._get_data()
                if self._data and name_in_j in self._data:
                    cache = self._data[name_in_j]
                else:
                    cache = func(self, *args, **kwargs)
            return cls(cache['id'], cache, self._session)

        return wrapper

    return wrappers_wrapper



# coding=utf-8

from __future__ import unicode_literals

from .base import Base
from .generator import generator_of
from .normal import normal_attr
from .streaming import streaming
from zhihu_oauth.zhcls.urls import (
    QUESTION_DETAIL_URL,
    QUESTION_ANSWERS_URL,
    QUESTION_COMMENTS_URL,
    QUESTION_FOLLOWERS_URL,
    QUESTION_TOPICS_URL,
)

__all__ = ['Question']


class Question(Base):
    def __init__(self, qid, cache, session):
        super(Question, self).__init__(qid, cache, session)

    def _build_url(self):
        return QUESTION_DETAIL_URL.format(self._id)

    # ----- simple info -----

    @property
    @normal_attr()
    def allow_delete(self):
        return None

    @property
    @normal_attr()
    def answer_count(self):
        return None

    @property
    @normal_attr()
    def comment_count(self):
        return None

    @property
    @normal_attr('except')
    def excerpt(self):
        """
        知乎返回的 json 里这一项叫做 except.... 也是醉了
        """
        return None

    @property
    @normal_attr()
    def follower_count(self):
        return None

    @property
    @normal_attr()
    def id(self):
        return self._id

    @property
    @normal_attr()
    def detail(self):
        return None

    @property
    @streaming()
    def redirection(self):
        """
        常见返回值：

        ..  code-block:: python

            {
                'to':
                {
                    'url': 'https://api.zhihu.com/questions/19570036',
                    'id': 19570036,
                    'type': 'question',
                    'title': '什么是「问题重定向」？如何正确使用该功能解决重复问题？'
                },
                'from':
                [
                    {
                        'url': 'https://api.zhihu.com/questions/19772082',
                        'id': 19772082,
                        'type': 'question',
                        'title': '知乎上有重复的问题吗？'
                    },
                    {
                        'url': 'https://api.zhihu.com/questions/20830682',
                        'id': 20830682,
                        'type': 'question',
                        'title': '各位知友以为同一问题重复出现，知乎应如何应对？'
                    }
                ]
            }

        在使用 from 属性时遇到语法错误？请看 :ref:`说明 <tips-for-conflict-with-keyword>`

        """
        return None

    @property
    @streaming()
    def status(self):
        return None

    @property
    @streaming(use_cache=False)
    def suggest_edit(self):
        """
        常见返回值：

        ..  code-block:: python

            {'status': False', reason': ''}

            {'status': True, 'reason': '问题表意不明'}
        """
        return None

    @property
    @normal_attr()
    def title(self):
        return None

    @property
    @normal_attr()
    def updated_time(self):
        return None

    # ----- generators -----

    @property
    @generator_of(QUESTION_ANSWERS_URL)
    def answers(self):
        return None

    @property
    @generator_of(QUESTION_COMMENTS_URL)
    def comments(self):
        return None

    @property
    @generator_of(QUESTION_FOLLOWERS_URL, 'people')
    def followers(self):
        return None

    @property
    @generator_of(QUESTION_TOPICS_URL)
    def topics(self):
        return None



# coding=utf-8

from __future__ import unicode_literals

from .base import Base
from .generator import generator_of
from .normal import normal_attr
from .streaming import streaming
from .urls import (
    PEOPLE_ACTIVITIES_URL,
    PEOPLE_DETAIL_URL,
    PEOPLE_ANSWERS_URL,
    PEOPLE_ARTICLES_URL,
    PEOPLE_COLLECTIONS_URL,
    PEOPLE_COLUMNS_URL,
    PEOPLE_FOLLOWERS_URL,
    PEOPLE_FOLLOWING_COLUMNS_URL,
    PEOPLE_FOLLOWING_QUESTIONS_URL,
    PEOPLE_FOLLOWING_TOPICS_URL,
    PEOPLE_FOLLOWINGS_URL,
    PEOPLE_QUESTIONS_URL,
)

__all__ = ['ANONYMOUS', 'People']


class _Anonymous(object):
    def __init__(self):
        self.id = 0
        self.name = '匿名用户'

    def __getattr__(self, _):
        # 匿名用户除了姓名以外所有属性均为 None
        return None


ANONYMOUS = _Anonymous()
"""
.. role:: py_code(code)
   :language: python

统一的匿名用户对象，可以使用 :py_code:`if people is ANONYMOUS:` 判断是否是匿名用户
"""


class People(Base):
    def __new__(cls, pid, cache, session):
        if pid == '0':
            return ANONYMOUS
        else:
            return super(People, cls).__new__(cls)

    def __init__(self, pid, cache, session):
        super(People, self).__init__(pid, cache, session)

    def _build_url(self):
        return PEOPLE_DETAIL_URL.format(self.id)

    # ---------- simple info ---------

    @property
    @normal_attr()
    def answer_count(self):
        return None

    @property
    @normal_attr()
    def articles_count(self):
        return None

    @property
    @normal_attr()
    def avatar_url(self):
        return None

    @property
    @streaming()
    def business(self):
        """
        用户所在行业。

        常见返回值：

        ..  code-block:: python

            {
                'introduction': '',
                'id': '19619368',
                'url': 'https://api.zhihu.com/topics/19619368',
                'type': 'topic',
                'avatar_url': 'http://pic1.zhimg.com/e82bab09c_s.jpg',
                'name': '计算机软件',
                'excerpt': '',
            }

        使用属性时必须先判断是否有效，如

        ..  code-block:: python

            if 'people.business:
                data = people.business.name

        """
        return {}

    @property
    @normal_attr('favorited_count')
    def collected_count(self):
        return None

    @property
    @normal_attr('favorite_count')
    def collection_count(self):
        return None

    @property
    @normal_attr('columns_count')
    def column_count(self):
        return None

    @property
    def columns_count(self):
        return self.column_count

    @property
    @normal_attr()
    def created_at(self):
        return None

    @property
    @normal_attr()
    def description(self):
        return None

    @property
    @normal_attr()
    def draft_count(self):
        return None

    @property
    @streaming()
    def educations(self):
        """
        教育信息。

        常见返回值：

        ..  code-block:: python

            [
                {
                    'major': {
                        'introduction': '计算机专业。<br>大众认为会是唯一会“修电脑”的专业。',
                        'id': '19639658',
                        'url': 'https://api.zhihu.com/topics/19639658',
                        'type': 'topic',
                        'avatar_url': 'http://pic2.zhimg.com/7e2fe4615_s.jpg',
                        'name': '计算机科学与技术',
                        'excerpt': '计算机专业。大众认为会是唯一会“修电脑”的专业。',
                    },
                    'school': {
                        'introduction': '',
                        'id': '1234567',
                        'url': 'https://api.zhihu.com/topics/1234567',
                        'type': 'topic',
                        'avatar_url': 'http://pic4.zhimg.com/8e6y3xd47_s.jpg',
                        'name': 'XX 大学',
                        'excerpt': '',
                    },
                },
            ]

        使用属性时必须先判断存不存在，如:

        ..  code-block:: python

            for education in people.educations:
                if 'school' in education:
                    data += education.school.name
                if 'major' in education:
                    data += education.major.name'
        """
        return []

    @property
    @streaming()
    def employments(self):
        """
        职业信息。

        常见返回值：

        ..  code-block:: python

            [
                {
                    'job': {
                        'introduction': '',
                        'url': 'https://api.zhihu.com/topics/19551336',
                        'avatar_url': 'http://pic3.zhimg.com/4eac47b76_s.jpg',
                        'excerpt': '',
                        'type': 'topic',
                        'name': '测试',
                        'id': '19551336',
                    },
                    'company': {
                        'excerpt': '',
                        'url': '',
                        'avatar_url': 'http://pic1.zhimg.com/e82bab09c_s.jpg',
                        'introduction': '',
                        'type': 'topic',
                        'name': 'Gayhub',
                        'experience ': '',
                        'id': '',
                    },
                },
            ],

        使用属性时必须先判断存不存在，如:

        ..  code-block:: python

            for employment in people.employments:
                if 'company' in education:
                    data += employment.company.name
                if 'job' in education:
                    data += employment.job.name'
        """
        return []

    @property
    @normal_attr()
    def email(self):
        return None

    @property
    def favorite_count(self):
        return self.collection_count

    @property
    def favorited_count(self):
        return self.collected_count

    @property
    @normal_attr()
    def follower_count(self):
        return None

    @property
    @normal_attr('following_columns_count')
    def following_column_count(self):
        return None

    @property
    @normal_attr()
    def following_count(self):
        return None

    @property
    @normal_attr()
    def following_question_count(self):
        return None

    @property
    @normal_attr()
    def following_topic_count(self):
        return None

    @property
    @normal_attr()
    def friendly_score(self):
        return None

    @property
    @normal_attr()
    def gender(self):
        """
        性别。

        =======  ==========
        值(int)  说明
        =======  ==========
        0        女
        1        男
        =======  ==========

        我该如何吐槽……
        """
        return None

    @property
    @normal_attr()
    def has_daily_recommend_permission(self):
        return None

    @property
    @normal_attr()
    def headline(self):
        """
        就是那个显示在名字后面的，和签名类似的东西。
        """
        return None

    @property
    @normal_attr()
    def is_active(self):
        return None

    @property
    @normal_attr()
    def id(self):
        return self._id

    @property
    @normal_attr()
    def is_baned(self):
        return None

    @property
    @normal_attr()
    def is_bind_sina(self):
        return None

    @property
    @normal_attr()
    def is_locked(self):
        return None

    @property
    @normal_attr()
    def is_moments_user(self):
        """
        不知道是啥。
        """
        return None

    @property
    @streaming()
    def locations(self):
        """
        常见返回值。

        ..  code-block:: python

            [
                {
                    'introduction': '天津，简称津，地处华北平原，balabala,
                    'url': 'https://api.zhihu.com/topics/19577238',
                    'avatar_url': 'http://pic4.zhimg.com/acad405e7_s.jpg',
                    'excerpt': '天津，简称津，地处华北平原 balabalabala',
                    'type': 'topic',
                    'name': '天津',
                    'id': '19577238',
                },
            ],

        使用属性时基本不用判断存不存在，如:

        ..  code-block:: python

            for location in people.locations:
                data += location.name
        """
        return []

    @property
    @normal_attr()
    def name(self):
        return None

    @property
    @normal_attr()
    def question_count(self):
        return None

    @property
    @normal_attr()
    def shared_count(self):
        return None

    @property
    @normal_attr()
    def sina_weibo_name(self):
        return None

    @property
    @normal_attr()
    def sina_weibo_url(self):
        return None

    @property
    @normal_attr()
    def thanked_count(self):
        return None

    @property
    @normal_attr()
    def uid(self):
        """
        没什么用的东西。
        """
        return None

    @property
    @normal_attr()
    def voteup_count(self):
        return None

    # ---------- generators ---------

    @property
    @generator_of(PEOPLE_ACTIVITIES_URL, 'activity')
    def activities(self):
        return None

    @property
    @generator_of(PEOPLE_ANSWERS_URL)
    def answers(self):
        return None

    @property
    @generator_of(PEOPLE_ARTICLES_URL)
    def articles(self):
        return None

    @property
    @generator_of(PEOPLE_COLLECTIONS_URL)
    def collections(self):
        return None

    @property
    @generator_of(PEOPLE_COLUMNS_URL)
    def columns(self):
        return None

    @property
    @generator_of(PEOPLE_FOLLOWERS_URL, 'people')
    def followers(self):
        """
        貌似知乎 API 有个限制，只允许获取前 5020 个粉丝，这好烦阿……
        """
        return None

    @property
    @generator_of(PEOPLE_FOLLOWING_COLUMNS_URL, 'column')
    def following_columns(self):
        return None

    @property
    @generator_of(PEOPLE_FOLLOWING_QUESTIONS_URL, 'question')
    def following_questions(self):
        return None

    @property
    @generator_of(PEOPLE_FOLLOWING_TOPICS_URL, 'topic')
    def following_topics(self):
        return None

    @property
    @generator_of(PEOPLE_FOLLOWINGS_URL, 'people')
    def followings(self):
        return None

    @property
    @generator_of(PEOPLE_QUESTIONS_URL)
    def questions(self):
        return None



# coding=utf-8

from .activity import Activity, ActType
from .answer import Answer
from .article import Article
from .collection import Collection
from .column import Column
from .comment import Comment
from .me import Me
from .people import People, ANONYMOUS
from .question import Question
from .topic import Topic

__all__ = ['Activity', 'ActType', 'Answer', 'Article', 'Collection', 'Column',
           'Comment', 'Me', 'People', 'ANONYMOUS', 'Question', 'Topic']



# coding=utf-8

from .client import ZhihuClient
from .zhcls import (
    Activity, ActType, Answer, Article, Comment, Column, Collection, People,
    Question, Topic, ANONYMOUS
)
from .exception import NeedCaptchaException, UnexpectedResponseException

__all__ = ['ZhihuClient', 'ANONYMOUS', 'Activity', 'ActType', 'Article',
           'Answer', 'Collection', 'Column', 'Comment', 'People', 'Question',
           'Topic', 'NeedCaptchaException', 'UnexpectedResponseException']

__version__ = '0.0.13'



# coding=utf-8

from __future__ import unicode_literals

import os

from ..exception import MyJSONDecodeError, UnexpectedResponseException

try:
    # Py3
    # noinspection PyCompatibility
    from html.parser import HTMLParser
except ImportError:
    # Py2
    # noinspection PyCompatibility,PyUnresolvedReferences
    from HTMLParser import HTMLParser

__all__ = ["INVALID_CHARS", "remove_invalid_char", 'add_serial_number',
           'SimpleHtmlFormatter']


def can_get_from(name, data):
    return name in data and not isinstance(data[name], (dict, list))


INVALID_CHARS = ['/', '\\', ':', '*', '?', '"', '<', '>', '|']


def remove_invalid_char(dirty, invalid_chars=None):
    invalid_chars = invalid_chars or INVALID_CHARS
    clean = []
    for c in dirty:
        if c not in invalid_chars:
            clean.append(c)
    return ''.join(clean)


def add_serial_number(file_path, postfix):
    full_path = file_path + postfix
    if not os.path.isfile(full_path):
        return full_path
    serial = 1
    while os.path.isfile(full_path):
        # noinspection PyUnboundLocalVariable
        try:
            # noinspection PyCompatibility,PyUnresolvedReferences
            serial_str = unicode(str(serial))
        except NameError:
            serial_str = str(serial)
        full_path = file_path + ' - ' + serial_str.rjust(3, '0') + '.' + postfix
        serial += 1
    return full_path


BASE_HTML_HEADER = """<meta name="referrer" content="no-referrer" />
<meta charset="utf-8" />
"""


# TODO: 测试 SimpleHtmlFormatter 对各种文章的兼容性


class SimpleHtmlFormatter(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self._level = 0
        self._last = ''
        self._in_code = False
        self._prettified = [BASE_HTML_HEADER]

    def handle_starttag(self, tag, attrs):
        if not self._in_code:
            self._prettified.extend(['\t'] * self._level)
        self._prettified.append('<' + tag)
        for name, value in attrs:
            self._prettified.append(' ' + name + '="' + value + '"')
        self._prettified.append('>')
        if not self._in_code:
            self._prettified.append('\n')
        if tag != 'br' and tag != 'img':
            self._level += 1
        if tag == 'code':
            self._in_code = True
        self._last = tag

    def handle_endtag(self, tag):
        if tag != 'br' and tag != 'img':
            self._level -= 1
        if not self._in_code:
            self._prettified.extend(['\t'] * self._level)
        self._prettified.append('</' + tag + '>')
        if not self._in_code:
            self._prettified.append('\n')
        self._last = tag
        if tag == 'code':
            self._in_code = False

    def handle_startendtag(self, tag, attrs):
        if not self._in_code:
            self._prettified.extend(['\t'] * self._level)
        self._prettified.append('<' + tag)
        for name, value in attrs:
            self._prettified.append(' ' + name + '="' + value + '"')
        self._prettified.append('/>')
        self._last = tag

    def handle_data(self, data):
        if not self._in_code:
            self._prettified.extend(['\t'] * self._level)
            if self._last == 'img':
                self._prettified.append('<br>\n')
                self._prettified.extend(['\t'] * self._level)
        self._prettified.append(data)
        if not self._in_code:
            self._prettified.append('\n')

    def handle_charref(self, name):
        self._prettified.append('&#' + name)

    def handle_entityref(self, name):
        self._prettified.append('&' + name + ';')

    def error(self, message):
        self._prettified = ['error when parser the html file.']

    def prettify(self):
        return ''.join(self._prettified)


class SimpleEnum(set):
    def __getattr__(self, item):
        if item in self:
            return item
        raise AttributeError("No {0} in this enum class.".format(item))


def get_result_or_error(url, res):
    try:
        json_dict = res.json()
        if 'error' in json_dict:
            return False, json_dict['error']['message']
        elif 'success' in json_dict:
            if json_dict['success']:
                return True, ''
            else:
                return False, 'Unknown error'
        else:
            return True, ''
    except (KeyError, MyJSONDecodeError):
        raise UnexpectedResponseException(
            url, res, 'a json contains voting result or error message')


def common_save(path, filename, content, default_filename, invalid_chars):
    filename = filename or default_filename
    filename = remove_invalid_char(filename, invalid_chars)
    filename = filename or 'untitled'

    path = path or '.'
    path = remove_invalid_char(path, invalid_chars)
    path = path or '.'

    if not os.path.isdir(path):
        os.makedirs(path)
    full_path = os.path.join(path, filename)
    full_path = add_serial_number(full_path, '.html')
    formatter = SimpleHtmlFormatter()
    formatter.feed(content)
    with open(full_path, 'wb') as f:
        f.write(formatter.prettify().encode('utf-8'))



# coding=utf-8

from __future__ import print_function, unicode_literals

import base64
import getpass
import os

import requests

from .oauth.before_login_auth import BeforeLoginAuth
from .oauth.zhihu_oauth import ZhihuOAuth
from .oauth.token import ZhihuToken
from .oauth.setting import (
    CAPTCHA_URL, LOGIN_URL, LOGIN_DATA, CLIENT_ID, APP_SECRET
)
from .oauth.utils import login_signature
from .setting import CAPTCHA_FILE, RE_FUNC_MAP, ADAPTER_WITH_RETRY
from .utils import need_login, int_id
from .exception import (
    UnexpectedResponseException, NeedCaptchaException, MyJSONDecodeError
)

__all__ = ['ZhihuClient']

try:
    # noinspection PyShadowingBuiltins,PyUnresolvedReferences
    input = raw_input
except NameError:
    pass

try:
    bs64decode = base64.decodebytes
except AttributeError:
    # for python 2
    # noinspection PyDeprecation
    bs64decode = base64.decodestring


class ZhihuClient:
    def __init__(self, client_id=None, secret=None):
        """
        知乎客户端，这是获取所有类的入口。

        :param str|unicode client_id: 客户端 ID。
        :param str|unicode secret: 客户端 ID 对应的 SECRET KEY。
        :rtype: :class:`.ZhihuClient`
        """
        self._session = requests.session()

        # Add auto retry for session
        self._session.mount('http://', ADAPTER_WITH_RETRY)
        self._session.mount('https://', ADAPTER_WITH_RETRY)

        # client_id and secret shouldn't have default value
        # after zhihu open api
        self._client_id = client_id or CLIENT_ID
        self._secret = secret or APP_SECRET

        self._login_auth = BeforeLoginAuth(self._client_id)
        self._token = None

    def need_captcha(self):
        """
        ..  note::

            一般来说此方法不需要手动调用。

            在调用 :meth:`.login` 时捕获 :class:`.NeedCaptchaException` 即可。

            而 :meth:`.login_in_terminal` 会自动处理需要验证码的情况。

        :return: 下次登录是否需要验证码。
        :rtype: bool
        :raise: :class:`.UnexpectedResponseException` 知乎返回的数据和预期格式不符。
        """
        res = self._session.get(CAPTCHA_URL, auth=self._login_auth)
        try:
            j = res.json()
            return j['show_captcha']
        except (MyJSONDecodeError, KeyError):
            # noinspection PyTypeChecker
            raise UnexpectedResponseException(
                CAPTCHA_URL, res,
                'a json data with show_captcha item'
            )

    def get_captcha(self):
        """
        :return: 如果需要验证码，则返回 bytes 型验证码，不需要则返回 None。
        :rtype: None | bytes
        :raise: :class:`.UnexpectedResponseException` 知乎返回的数据和预期格式不符
        """
        if self.need_captcha():
            res = self._session.put(CAPTCHA_URL, auth=self._login_auth)
            try:
                j = res.json()
                return bs64decode(j['img_base64'].encode('utf-8'))
            except (MyJSONDecodeError, ValueError, KeyError):
                raise UnexpectedResponseException(
                    CAPTCHA_URL,
                    res,
                    'a json string contain a img_base64 item.'
                )
        return None

    def login(self, username, password, captcha=None):
        """
        登录知乎的主要方法。

        ..  warning:: 关于手机号登录

            手机号登录时请在手机号前加上 ``+86``

        :param str|unicode username: 邮箱或手机号。
        :param str|unicode password: 密码。
        :param str|unicode captcha: 验证码，可以为空。

        :return: 二元元组，第一个元素表示是否成功，第二个元素表示失败原因。
        :rtype: tuple(bool, str)
        :raise: :class:`.NeedCaptchaException` 此次登录需要验证码
        """

        if captcha is None:
            try:
                if self.need_captcha():
                    raise NeedCaptchaException
            except UnexpectedResponseException as e:
                return False, str(e)
        else:
            res = self._session.post(
                CAPTCHA_URL,
                auth=self._login_auth,
                data={'input_text': captcha}
            )
            try:
                json_dict = res.json()
                if 'error' in json_dict:
                    return False, json_dict['error']['message']
            except (MyJSONDecodeError, ValueError, KeyError) as e:
                return False, str(e)

        data = dict(LOGIN_DATA)
        data['username'] = username
        data['password'] = password
        data['client_id'] = self._client_id

        login_signature(data, self._secret)
        res = self._session.post(LOGIN_URL, auth=self._login_auth, data=data)
        try:
            json_dict = res.json()
            if 'error' in json_dict:
                return False, json_dict['error']['message']
            else:
                self._token = ZhihuToken.from_dict(json_dict)
                self._session.auth = ZhihuOAuth(self._token)
                return True, ''
        except (MyJSONDecodeError, ValueError, KeyError) as e:
            return False, str(e)

    def login_in_terminal(self, username=None, password=None):
        """
        为在命令行模式下使用本库的用户提供的快捷登录方法。

        在未提供 username 或 password 参数时会在终端中请求输入。

        ..  note:: 此方法会自动处理验证码需要验证码情况。

        :param str|unicode username: 邮箱或手机号。
        :param str|unicode password: 密码。
        :return: .. seealso:: :meth:`.login`
        """
        print('----- Zhihu OAuth Login -----')
        username = username or input('email: ')
        password = password or getpass.getpass('password: ')

        try:
            success, reason = self.login(username, password)
        except NeedCaptchaException:
            print('Need for a captcha, getting it......')
            captcha_image = self.get_captcha()
            with open('captcha.gif', 'wb') as f:
                f.write(captcha_image)
            print('Please open {0} for captcha'.format(
                os.path.abspath(CAPTCHA_FILE)))
            captcha = input('captcha: ')
            os.remove(os.path.abspath(CAPTCHA_FILE))
            success, reason = self.login(username, password, captcha)
        if success:
            print('Login success.')
        else:
            print('Login failed, reason: ', reason)

        return success, reason

    def create_token(self, filename, username=None, password=None):
        """
        另一个快捷方法，作用为调用 :meth:`.login_in_terminal`

        如果成功则将 token 储存文件中。

        :param str|unicode filename: token 保存的文件名
        :param str|unicode username: 邮箱或手机号
        :param str|unicode password: 密码
        :return: .. seealso:: :meth:`.login`
        """
        success, reason = self.login_in_terminal(username, password)
        if success:
            self.save_token(filename)
            print('Token file created success.')
        else:
            print('Token file created failed.')
        return success, reason

    def load_token(self, filename):
        """
        通过载入 token 文件来达到登录状态。

        ..  seealso:: :meth:`.save_token`

        :param str|unicode filename: token 文件名。
        :return: 无返回值，也就是说其实不知道是否登录成功。
        """
        self._token = ZhihuToken.from_file(filename)
        self._session.auth = ZhihuOAuth(self._token)

    @need_login
    def save_token(self, filename):
        """
        将通过登录获取到的 token 保存为文件，必须是已登录状态才能调用。

        ..  seealso:: :meth:`.load_token`

        :param str|unicode filename: 将 token 储存为文件。
        :return: 无返回值。
        """
        self._token.save(filename)

    def is_login(self):
        """
        :return: 是否已登录。但其实只是检查内部的 token 是否是 None。
        :rtype: bool
        """
        return self._token is not None

    @need_login
    def test_api(self, method, url, params=None, data=None):
        """
        开发时用的测试某个 API 返回的 JSON 用的便捷接口。

        :param str|unicode method: HTTP 方式， GET or POST or OPTION, etc。
        :param str|unicode url: API 地址。
        :param dict params: GET 参数。
        :param dict data: POST 参数。
        :return: 访问结果。
        :rtype: request.Response
        """
        return self._session.request(method, url, params, data)

    def set_proxy(self, proxy):
        """ 设置 http 和 https 代理（不支持 socks 代理）

        因为由 :any:`ZhihuClient` 生成的知乎类对象和本对象使用同一
        session，所以设置代理后，对所有由当前对象生成的知乎对象均会
        使用设置的代理。

        :param str|unicode proxy: 形如 '10.10.1.10:3128'，传入 None
          表示清除代理设置。
        :return: None
        """
        if proxy is None:
            self._session.proxies.clear()
        else:
            self._session.proxies.update({'http': proxy, 'https': proxy})

    # ----- get zhihu classes from ids -----

    @int_id
    @need_login
    def answer(self, aid):
        """
        获取答案对象，需要 Client 是登录状态。

        :param int aid: 答案 ID。
        :举例:
            https://www.zhihu.com/question/xxxxxx/answer/1234567
            的答案 ID 是 1234567。
        :rtype: Answer
        """
        from .zhcls.answer import Answer
        return Answer(aid, None, self._session)

    @int_id
    @need_login
    def article(self, aid):
        """
        获取文章对象，需要 Client 是登录状态。

        :param int aid: 文章 ID。
        :举例: https://zhuanlan.zhihu.com/p/1234567 的文章 ID 是 1234567。
        :rtype: :class:`Article`
        """
        from .zhcls.article import Article
        return Article(aid, None, self._session)

    @int_id
    @need_login
    def collection(self, cid):
        """
        获取收藏夹对象，需要 Client 是登录状态。

        :param int cid: 收藏夹 ID
        :举例: https://www.zhihu.com/collection/1234567 的收藏夹 ID 是 1234567。
        :rtype: :class:`Collection`
        """
        from .zhcls.collection import Collection
        return Collection(cid, None, self._session)

    @need_login
    def column(self, cid):
        """
        获取专栏对象，需要 Client 是登录状态。

        :param str|unicode cid: 专栏 ID，注意，类型是字符串。
        :举例: https://zhuanlan.zhihu.com/abcdefg 的专栏 ID 是 abcdefg。
        :rtype: :class:`Column`
        """
        from .zhcls.column import Column
        return Column(cid, None, self._session)

    @need_login
    def me(self):
        """
        获取当前登录的用户，需要 Client 是登录状态。

        ..  note::
            :class:`Me` 类继承于 :class:`People`，是一个不同于其他用户的类。
            这个类用于提供各种操作，比如点赞，评论，私信等。

        :rtype: :class:`Me`
        """

        from .zhcls import Me
        return Me(self._token.user_id, None, self._session)

    @need_login
    def people(self, pid):
        """
        获取用户对象，需要 Client 是登录状态。

        :param str|unicode pid: 用户 ID，注意，类型是字符串。
        :举例: https://www.zhihu.com/people/abcdefg 的用户 ID 是 abcdefg。
        :rtype: :class:`Column`
        """
        from .zhcls.people import People
        return People(pid, None, self._session)

    @int_id
    @need_login
    def question(self, qid):
        """
        获取问题对象，需要 Client 是登录状态。

        :param int qid: 问题 ID。
        :举例: https://www.zhihu.com/question/1234567 的问题 ID 是 1234567。
        :rtype: :class:`Question`
        """
        from .zhcls.question import Question
        return Question(qid, None, self._session)

    @int_id
    @need_login
    def topic(self, tid):
        """
        获取话题对象，需要 Client 是登录状态。

        :param int tid: 话题 ID。
        :举例: https://www.zhihu.com/tipoc/1234567 的话题 ID 是 1234567。
        :rtype: :class:`Topic`
        """
        from .zhcls.topic import Topic
        return Topic(tid, None, self._session)

    @need_login
    def from_url(self, url):
        """
        通过知乎的 URL 创建对象，需要 client 是登录状态。

        对象的 URL 请参见对应方法的描述。如 :any:`Answer` 类的 URL 的描述在
        :any:`ZhihuClient.answer` 方法的文档里，其余类似。

        ..  note:: 提示

            本方法也支持省略了开头的 ``https://``，或者结尾有多余的 ``/`` 的 URL。

        :param str|unicode url: 知乎对象的网址
        :return: 对应的知乎对象
        """
        for re, val in RE_FUNC_MAP.items():
            match = re.match(url)
            if match:
                zhihu_obj_id = match.group(1)
                func_name, need_int_id = val
                if need_int_id:
                    zhihu_obj_id = int(zhihu_obj_id)
                return getattr(self, func_name)(zhihu_obj_id)
        raise ValueError('Invalid zhihu object url!')
